import json
import socket
from typing import List, Optional

from aiwolf.gameinfo import GameInfo, _GameInfo
from aiwolf.gamesetting import GameSetting, _GameSetting
from aiwolf.utterance import Talk, Whisper, _Utterance


class Parser:
    """parse for json, this is active by default in aiwolf-python"""

    def __init__(self):
        self.game_info: Optional[GameInfo] = None
        self.last_game_info: Optional[GameInfo] = None
        self.game_setting: Optional[GameSetting] = None

    def initialize(self, game_info0: Optional[_GameInfo], game_setting0: Optional[_GameSetting]):
        self.game_info = GameInfo(game_info0) if game_info0 is not None else None
        if self.game_info is None:
            self.game_info = self.last_game_info
        else:
            self.last_game_info = self.game_info
        if self.game_info is None:
            return None

        self.game_setting = GameSetting(game_setting0)
        return None


    def update(
            self, game_info0: Optional[_GameInfo],
            talk_history0: Optional[List[_Utterance]], whisper_history0: Optional[List[_Utterance]],
            request: str
    ):
        self.game_info = GameInfo(game_info0) if game_info0 is not None else None
        if self.game_info is None:
            self.game_info = self.last_game_info
        else:
            self.last_game_info = self.game_info
        if self.game_info is None:
            return None

        if talk_history0 is not None:
            for talk0 in talk_history0:
                talk: Talk = Talk.compile(talk0)
                talk_list: List[Talk] = self.game_info.talk_list
                if len(talk_list) == 0:
                    talk_list.append(talk)
                else:
                    last_talk: Talk = talk_list[-1]
                    if talk.day > last_talk.day or (talk.day == last_talk.day and talk.idx > last_talk.idx):
                        talk_list.append(talk)

        if whisper_history0 is not None:
            for whisper0 in whisper_history0:
                whisper: Whisper = Whisper.compile(whisper0)
                whisper_list: List[Whisper] = self.game_info.whisper_list
                if len(whisper_list) == 0:
                    whisper_list.append(whisper)
                else:
                    last_whisper: Whisper = whisper_list[-1]
                    if whisper.day > last_whisper.day or (whisper.day == last_whisper.day and whisper.idx > last_whisper.idx):
                        whisper_list.append(whisper)

            return None

