# Player class for primitive implements

from abc import ABC, abstractmethod
from typing import Optional, List

from aiwolf.gameinfo import _GameInfo
from aiwolf.gamesetting import _GameSetting
from aiwolf.utterance import _Utterance


class AbstractPlayer(ABC):
    """Abstract class that defines the functions every player agents must have."""

    @abstractmethod
    def attack(self) -> int:
        pass

    @abstractmethod
    def day_start(self) -> None:
        pass

    @abstractmethod
    def divine(self) -> int:
        pass

    @abstractmethod
    def finish(self) -> None:
        pass

    @abstractmethod
    def guard(self) -> int:
        pass

    @abstractmethod
    def initialize(self, game_info: _GameInfo, game_setting: _GameSetting) -> None:
        pass

    @abstractmethod
    def talk(self) -> str:
        pass

    @abstractmethod
    def update(
            self, game_info: _GameInfo, talk_history: Optional[List[_Utterance]],
            whisper_history: Optional[List[_Utterance]], request: str
    ) -> None:
        pass

    @abstractmethod
    def vote(self) -> int:
        pass

    @abstractmethod
    def whisper(self) -> str:
        pass
