from typing import Optional, List
from collections import deque
import random

from aiwolf import (Agent, ComingoutContentBuilder, Content, GameInfo, GameSetting,
                    DivinedResultContentBuilder, IdentContentBuilder, Judge, Role, Species, Status,
                    VoteContentBuilder, AttackContentBuilder, SkipContentBuilder, OverContentBuilder)
from aiwolf.gameinfo import GameInfo, _GameInfo
from aiwolf.gamesetting import GameSetting, _GameSetting
from aiwolf.utterance import Talk, Whisper, _Utterance
from aiwolf.constant import AGENT_NONE
from aiwolfpy.player import AbstractPlayer
from aiwolfpy.parser import Parser


class SamplePlayer(AbstractPlayer):
    
    def __init__(self):
        self.parser = Parser()
        self.me = None
        self.role = Role.UNC
        self.co = Role.UNC
        self.werewolves = []
        self.humans = []
        self.talk_list_head = 0
        self.my_judge_queue = deque()
        self.vote_target = AGENT_NONE
        self.vote_declare = AGENT_NONE
        self.attack_vote_target = AGENT_NONE
        self.attack_vote_declare = AGENT_NONE
        self.divine_target = AGENT_NONE
        self.guard_target = AGENT_NONE

    def initialize(self, game_info: _GameInfo, game_setting: _GameSetting):
        print("---------")
        print("")
        print("---------")
        print("INITIALIZE")
        print(game_info)
        print(game_setting)
        # initialize parser
        self.parser.initialize(game_info, game_setting)
        game_info = self.parser.game_info

        # my role
        self.me = game_info.me
        self.role = game_info.my_role
        self.co = Role.UNC
        self.werewolves = []
        self.humans = []
        self.talk_list_head = 0
        self.my_judge_queue.clear()
        self.vote_target = self.me
        self.vote_declare = AGENT_NONE
        self.attack_vote_target = self.me
        self.attack_vote_declare = AGENT_NONE
        self.divine_target = self.me
        self.guard_target = self.me

        # for werewolves
        if self.role == Role.WEREWOLF:
            self.werewolves = list(game_info.role_map.keys())
            self.humans = [a for a in game_info.agent_list if a not in self.werewolves]

    def update(
            self, game_info: _GameInfo, talk_history: Optional[List[_Utterance]],
            whisper_history: Optional[List[_Utterance]], request
    ):
        print(request)
        print(game_info)
        print(talk_history)
        print(whisper_history)
        self.parser.update(game_info, talk_history, whisper_history, request)

    def day_start(self):
        self.talk_list_head = 0
        self.vote_target = self.me
        self.vote_declare = AGENT_NONE
        self.attack_vote_target = self.me
        self.attack_vote_declare = AGENT_NONE
        self.divine_target = self.me
        self.guard_target = self.me

        # get result
        # SEER
        if self.role == Role.SEER:
            judge: Optional[Judge] = self.parser.game_info.divine_result
            if judge is not None:
                self.my_judge_queue.append(judge)
        # MEDIUM
        if self.role == Role.MEDIUM:
            judge: Optional[Judge] = self.parser.game_info.medium_result
            if judge is not None:
                self.my_judge_queue.append(judge)
        # FAKE SEER RESULT
        if self.role != self.co == Role.SEER:
            # random target
            candidate_list = []
            for a in self.parser.game_info.agent_list:
                if a == self.me:
                    continue
                if self.parser.game_info.status_map[a] == Status.ALIVE:
                    candidate_list.append(a)
            for a in self.parser.game_info.last_dead_agent_list:
                candidate_list.append(a)
            target: Agent = random.choice(candidate_list)
            # random result
            result: Species = Species.HUMAN
            d = 0.5
            if random.random() < d and target not in self.parser.game_info.last_dead_agent_list:
                result = Species.WEREWOLF
            judge = Judge(self.me, self.parser.game_info.day, target, result)
            self.my_judge_queue.append(judge)

        return None
    
    def talk(self):
        # co anyway
        if self.co == Role.UNC:
            if self.role == Role.SEER:
                self.co = Role.SEER
                content = Content(ComingoutContentBuilder(target=self.me, role=Role.SEER))
                return content.text
            elif self.role == Role.MEDIUM:
                self.co = Role.MEDIUM
                content = Content(ComingoutContentBuilder(target=self.me, role=Role.MEDIUM))
                return content.text
            elif self.role == Role.POSSESSED:
                # co as seer
                self.co = Role.SEER
                content = Content(ComingoutContentBuilder(target=self.me, role=Role.SEER))
                # add fake result for day 1
                # random target
                target: Agent = random.choice([
                    a for a in self.parser.game_info.agent_list if a != self.me
                ])
                # random result
                result: Species = Species.HUMAN
                num_wolves = self.parser.game_setting.role_num_map.get(Role.WEREWOLF, 0)
                d = num_wolves / (self.parser.game_setting.player_num - 1)
                if random.random() < d:
                    result = Species.WEREWOLF
                judge = Judge(self.me, self.parser.game_info.day, target, result)
                self.my_judge_queue.append(judge)
                return content.text
            else:
                self.co = Role.VILLAGER
        # what about bodyguard? powerplay?

        # result
        if len(self.my_judge_queue) > 0:
            if self.role == self.co == Role.SEER:
                judge: Judge = self.my_judge_queue.popleft()
                content = Content(DivinedResultContentBuilder(judge.target, judge.result))
                return content.text
            elif self.role == self.co == Role.MEDIUM:
                judge: Judge = self.my_judge_queue.popleft()
                content = Content(IdentContentBuilder(judge.target, judge.result))
                return content.text
            elif self.co == Role.SEER:
                judge: Judge = self.my_judge_queue.popleft()
                content = Content(DivinedResultContentBuilder(judge.target, judge.result))
                return content.text
        # think who to vote for
        # think once because it is a sample
        if self.vote_declare == AGENT_NONE:
            self._think_vote()

        # declare who to vote for
        if self.vote_declare is AGENT_NONE or self.vote_declare != self.vote_target:
            self.vote_declare = self.vote_target
            content = Content(VoteContentBuilder(target=self.vote_declare))
            return content.text

        # something to talk later
        # if False:
        #     content = Content(SkipContentBuilder())
        #     return content.text

        # nothing to talk anymore
        content = Content(OverContentBuilder())
        return content.text

    def whisper(self):
        if self.co == Role.UNC:
            self.co = Role.VILLAGER
            content = Content(ComingoutContentBuilder(target=self.me, role=Role.VILLAGER))
            return content.text

        if self.parser.game_info.day > 0:
            # think who to attack
            # think once because it is a sample
            if self.attack_vote_declare == AGENT_NONE:
                self._think_attack_vote()
            # declare who to attack
            if self.attack_vote_declare is AGENT_NONE or self.attack_vote_declare != self.attack_vote_target:
                self.attack_vote_declare = self.attack_vote_target
                content = Content(AttackContentBuilder(target=self.attack_vote_declare))
                return content.text
        # something to talk later
        # if False:
        #     content = Content(SkipContentBuilder())
        #     return content.text

        # nothing to talk anymore
        content = Content(OverContentBuilder())
        return content.text
        
    def vote(self):
        return self.vote_target.agent_idx
    
    def attack(self):
        return self.attack_vote_target.agent_idx
    
    def divine(self):
        # random target
        candidate_list = []
        for a in self.parser.game_info.agent_list:
            if a == self.me:
                continue
            if self.parser.game_info.status_map[a] == Status.ALIVE:
                candidate_list.append(a)
        self.divine_target = random.choice(candidate_list)
        return self.divine_target.agent_idx
    
    def guard(self):
        # random target
        candidate_list = []
        for a in self.parser.game_info.agent_list:
            if a == self.me:
                continue
            if self.parser.game_info.status_map[a] == Status.ALIVE:
                candidate_list.append(a)
        self.guard_target = random.choice(candidate_list)
        return self.guard_target.agent_idx
    
    def finish(self):
        # you can update your AI if you want and can
        return None

    def _think_vote(self):
        # random target
        candidate_list = []
        for a in self.parser.game_info.agent_list:
            if a == self.me:
                continue
            # do not vote friends if werewolf
            if a in self.werewolves and self.role == Role.WEREWOLF:
                continue
            if self.parser.game_info.status_map[a] == Status.ALIVE:
                candidate_list.append(a)
        self.vote_target = random.choice(candidate_list)
        return None

    def _think_attack_vote(self):
        # random target in alive human
        candidate_list = []
        for a in self.humans:
            if self.parser.game_info.status_map[a] == Status.ALIVE:
                candidate_list.append(a)
        self.attack_vote_target = random.choice(candidate_list)
        return None
